import sys
import os
import re
import math

from PyQt5 import QtWidgets, QtGui, QtCore    #, uic, QtQml, QtQuick
from PyQt5.QtWidgets import QFileDialog, QMessageBox, QDesktopWidget
from PyQt5.QtCore import Qt, QTimer

import pyqtgraph as pg
import json

import datetime, time

#import GroupsFrame, LabelFrame, iws_defs #, GraphFrame

import GraphFrame
###import GraphFrame_alt

import iws_defs

# Parent is HkMonitorWindow
class MultiPlotWindow(QtWidgets.QMainWindow):

    def __init__(self, config_file, which_monitor, parent=None):
        super(MultiPlotWindow, self).__init__(parent)

        self.parent = parent
        self.graph_frames = []
        self.name = ''
        self.labels = {}
        self.on_display = which_monitor

        self.layout(config_file)


    def do_close(self):
        self.parent.child_plot_window_closed(self.name)
        self.close()


    def layout(self, config_file):

        frame = QtWidgets.QFrame()
        layout = QtWidgets.QVBoxLayout()

        with open(config_file) as f:
            json_data = json.load(f)

        name = json_data['name']

        self.scroll = QtWidgets.QScrollArea()             
        self.scroll.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        self.scroll.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        self.scroll.setWidgetResizable(True)
        #self.scroll.setWidget(self.widget)

        pframe = self.make_plot_frame(json_data)
        bframe = self.make_button_frame()


        #pframe = self.make_frame(json_data['graphs'])
        #bframe = self.make_button_frame()
        #layout = QtWidgets.QVBoxLayout()
        layout.addWidget(bframe)
        layout.addWidget(pframe)

        widget = QtWidgets.QWidget()
        widget.setLayout(layout)
#        self.setCentralWidget(widget)
        self.setWindowTitle(name)
        self.name = name

        self.scroll.setWidget(widget)
        self.setCentralWidget(self.scroll)

        """
        self.scroll = QtWidgets.QScrollArea()             
        self.scroll.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        self.scroll.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        self.scroll.setWidgetResizable(True)
        #self.scroll.setWidget(self.widget)
        self.scroll.setWidget(widget)
        self.setCentralWidget(self.scroll)
        """

        ###self.parent.all_mplot_labels.update(self.labels)
        #self.debug()
        #self.show()

        #display_monitor = 1 # the number of the monitor you want to display your widget, starting from 0

        monitor = QDesktopWidget().screenGeometry(self.on_display)
        self.move(monitor.left(), monitor.top())
        self.showMaximized()


    def debug(self):
        print('MultiPlotWindow labels:')
        print(f'{self.labels}')


    def make_button_frame(self):
        frame = QtWidgets.QFrame()
        self.close_button = QtWidgets.QPushButton('Close')
        self.close_button.clicked.connect(self.do_close)
        layout = QtWidgets.QHBoxLayout()
        layout.addStretch(1)
        layout.addWidget(self.close_button)
        frame.setLayout(layout)
        return frame


    def make_frame(self, n, json_data):
        #frame = QtWidgets.QFrame()
        #print(f"MultiPlotWindow: make_frame : n = {n}, json = {json_data}")
        data = json_data[n]
        name = data['name']
        frame = GraphFrame.GraphFrame(name, data, legend=False, parent = self.parent)
########        frame = GraphFrame_alt.GraphFrame_alt(name, data, legend=False, parent = self.parent)
        #frame.setStyleSheet("background-color:blue;")
        self.labels.update(frame.parameters())
        #print(f'MultiPlotWindow make_frame: parameters = {frame.parameters()}')
        self.parent.ui.all_mplot_labels.update(frame.parameters())

        if 'row' in data:
            row = data['row']
            col = data['col']
            print(f'row = {row}, col = {col}')
        else:
            row = None
            col = None

        return (frame, row, col)

    
    def make_plot_frame(self, json_data):

        #glw = pg.GraphicsLayoutWidget(parent=self, show=True, title='Foo', border = True)
        #plots = []

        n = json_data['ngraphs']

        frame = QtWidgets.QFrame()
        layout = QtWidgets.QGridLayout()

        for i in range(n):

            (gframe, r, c) = self.make_frame(i, json_data['graphs'])
            if r != None:
                layout.addWidget(gframe, r, c)
            else:
                row = int(math.floor(i / 2))
                col = i % 2
                layout.addWidget(gframe, row, col)

            self.graph_frames.append(gframe)

        #return glw

        frame.setLayout(layout)
        return frame


    '''
    def make_parameter_frame(self, jdata):
        tab_frame = QtWidgets.QFrame()
        tab_widget = QtWidgets.QTabWidget()

        for i in jdata:
            tab_name = i['name']
            ltab = GraphFrame.GraphFrame(tab_name, i, legend=False)
            ltab.group_name = tab_name
            #ltab.setStyleSheet("background-color:yellow")
            tab_widget.addTab(ltab, tab_name)
            self.graph_frames.append(ltab)
            ltab.debug()
            self.labels.update(ltab.parameters())
            self.parent.ui.all_mplot_labels.update(ltab.parameters())

        #print(f'MultiPlotWindow make_parameter_frame: parent labels = {self.parent.ui.all_mplot_labels}')
        layout = QtWidgets.QHBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(tab_widget)
        tab_frame.setLayout(layout)
        return tab_frame
    '''


    def update_plot(self, obt_time, time_conversion_string, xval):
        #get_time = datetime.datetime.strptime(obt_time, time_conversion_string)
        #xaxis_time_str = str(get_time.hour) + ":" + str(get_time.minute) + ":" + str(get_time.second)
        #xval = int(time.mktime(get_time.timetuple()))
        for frame in self.graph_frames:
            #No time entry for upddate_plot so see if we
            #have a a previous saved time being used.
            frame.update_plot(obt_time, time_conversion_string, xval)


    '''Update the tm parameters but as they are not visible do not update their display
    '''
    def update_tm(self, parameter_string):
        #print(f'{self.name} update_tm')
        mib_id = parameter_string[iws_defs.TM_MIB_ID_INDEX]

        try:
            if mib_id in self.labels:
                self.labels[mib_id].update_tm(parameter_string, display = False)
        except Exception as e:
            print(f'Exception in MultiPlotWindow update_tm: {e}')
            #if mib_id == 'VOGT2856':
                ##print(f'[{mib_id}] caused exception in update_tm in HkMonitorUi all_labels')
                #pass
            pass

#        for frame in self.graph_frames:
#            frame.update_tm(parameter_string)
#            #if frame.update_tm(parameter_string):
#                #break
